#!/usr/bin/env node

/* eslint-disable no-console */
'use strict' // eslint-disable-line

var spawn = require('cross-spawn') // eslint-disable-line
var chalk = require('chalk') // eslint-disable-line
var semver = require('semver') // eslint-disable-line

// 检测Node.js版本前运行代码纯ES5
var packageJson = require('./package.json') // eslint-disable-line

if (!semver.satisfies(process.version, packageJson.engines.node)) {
  console.error(
    chalk.red('You are running Node %s.\n' +
        'Create Web Scripts App requires Node %s or higher. \n' +
        'Please update your version of Node.'),
    process.version,
    packageJson.engines.node,
  )
  process.exit(1)
}

// process.argv 属性返回一个数组，这个数组包含了启动Node.js进程时的命令行参数
// 第一个元素为process.execPath。如果需要获取argv[0]的值请参见 process.argv0。
// 第二个元素为当前执行的JavaScript文件路径(process.cwd())。剩余的元素为其他命令行参数。
const args = process.argv.slice(2)

const SCRIPTS = ['build', 'start']

const scriptIndex = args.findIndex(x => SCRIPTS.includes(x))
const script = args.find(x => SCRIPTS.includes(x))
if (script) {
  // 传递参数执行脚本指令
  const result = spawn.sync(
    'node',
    args.fill(require.resolve(`./${script}`), scriptIndex, scriptIndex + 1),
    { stdio: 'inherit' },
  )
  if (result.signal) {
    if (result.signal === 'SIGKILL' || result.signal === 'SIGTERM') {
      console.log(chalk.red('Process has been end'))
    }
    process.exit(1)
  }
  process.exit(result.status)
} else {
  console.log(chalk.red(`Unknow script "${script}".`))
}
