/* eslint-disable import/no-unresolved, max-params, global-require */

const fs = require('fs-extra')
const path = require('path')
const chalk = require('chalk')
const spawn = require('cross-spawn')
const { execSync } = require('child_process')

const dependencies = [
  'history',
  'prop-types',
  'react',
  'react-dom',
  'react-redux',
  'react-router',
  'react-router-dom',
  'react-router-redux@next',
  'redux',
  'redux-thunk',
]

function shouldUse() {
  try {
    execSync('cnpm --version', { stdio: 'ignore' })
    return {
      command: 'cnpm',
      args: ['install', '-S'].concat(dependencies),
    }
  } catch (e) {
    try {
      execSync('yarn --version', { stdio: 'ignore' })
      return {
        command: 'yarn',
        args: ['add'].concat(dependencies),
      }
    } catch (err) {
      return {
        command: 'npm',
        args: ['install', '-S'].concat(dependencies),
      }
    }
  }
}

/**
 * react项目初始化
 * @param  {string} appPath           被构建的项目目录
 * @param  {string} appName           被构建的项目名
 */
function init(appPath, appName) {
  const ownPackageName = require(path.join(__dirname, 'package.json')).name
  const ownPath = path.join(appPath, 'node_modules', ownPackageName)
  const appPackage = require(path.join(appPath, 'package.json'))

  // Copy over some of the devDependencies
  appPackage.dependencies = appPackage.dependencies || {}

  // Setup the script rules
  appPackage.scripts = {
    start: `${ownPackageName} start`,
    build: `${ownPackageName} build`,
    // test: `${ownPackageName} test --env=jsdom`,
    // eject: `${ownPackageName} eject`,
  }

  // 对象补充赋值属性，然后进行赋值
  fs.writeFileSync(
    path.join(appPath, 'package.json'),
    JSON.stringify(appPackage, null, 2),
  )

  // Copy the files for the user
  const templatePath = path.join(ownPath, 'template')
  fs.copySync(templatePath, appPath)

  // Rename gitignore after the fact to prevent npm from renaming it to .npmignore
  // See: https://github.com/npm/npm/issues/1862
  fs.move(
    path.join(appPath, 'gitignore'),
    path.join(appPath, '.gitignore'),
  ).then(console.error)


  // 安装react全家桶
  const { args, command } = shouldUse()
  // Install react, react-dom, react-router and so on
  const proc = spawn.sync(command, args, { stdio: 'inherit' })
  if (proc.status !== 0) {
    console.error(chalk.red(`\n\n${command} ${args.join(' ')} failed`))
    return
  }

  console.log(chalk.green(`Success! Created ${chalk.yellow(appName)} at ${chalk.yellow(appPath)}`))
}

module.exports = init
