/* eslint-disable import/no-unresolved, no-console */
'use strict' // eslint-disable-line

process.env.NODE_ENV = 'development'

// 脚本crash时，防止rejections静默处理
process.on('unhandledRejection', (err) => {
  throw err
})

// 确保环境变量已经被读取.
require('config-ivan')()

const chalk = require('chalk')
const url = require('url')
const webpack = require('webpack')
const WebpackDevServer = require('webpack-dev-server')

const config = require('./webpack.config.dev')
const serverConfig = require('./webpack.config.devserver')
const choosePort = require('./utils/choosePort')
const openBrowser = require('./utils/openBrowser')

// 环境变量获取端口、主机ip、协议
const DEFAULT_PORT = parseInt(process.env.PORT, 10) || 40000
const HOST = process.env.HOST || 'localhost'
const PROTOCOL = process.env.HTTPS === 'true' ? 'https' : 'http'

choosePort(HOST, DEFAULT_PORT)
  .then((port) => {
    // 创建webpack compiler.
    const compiler = webpack(config)

    const devServer = new WebpackDevServer(compiler, serverConfig)

    devServer.listen(port, HOST, (err) => {
      if (err) {
        return console.log(err)
      }
      const browserUrl = url.format({
        protocol: PROTOCOL, hostname: HOST, port, pathname: '/',
      })
      console.log(chalk.cyan(`启动开发服务器...${browserUrl}\n`))
      return openBrowser(browserUrl)
    });

    // 接收到系统信号SIGINT时触发，主要是用户按Ctrl + c时触发
    // SIGTERM信号表示内核要求当前进程停止
    ['SIGINT', 'SIGTERM'].forEach((sig) => {
      process.on(sig, () => {
        devServer.close()
        process.exit()
      })
    })
  })
  .catch((err) => {
    if (err && err.message) {
      console.log(chalk.red(err.message))
    }
    process.exit(1)
  })
