import React from 'react'
import { render } from 'react-dom'
import { createStore, applyMiddleware, compose } from 'redux'
import { Provider } from 'react-redux'
import createHistory from 'history/createBrowserHistory'
import { ConnectedRouter, routerMiddleware } from 'react-router-redux'
import thunk from 'redux-thunk'

import './index.css'
import reducers from './reducers/index'
import Router from './router'
// 请求要统一处理，please modify the middleware
import httpMiddleware from './middleware/httpmiddleware'
import registerServiceWorker from './registerServiceWorker'

const history = createHistory()
const middleware = [thunk, routerMiddleware(history), httpMiddleware]
// see: http://zalmoxisus.github.io/redux-devtools-extension/
// eslint-disable-next-line no-underscore-dangle
const composeEnhancers = window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ ?
  // eslint-disable-next-line no-underscore-dangle
  window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__({}) : compose

const enhancer = composeEnhancers(applyMiddleware(...middleware))

const store = createStore(reducers, enhancer)

render(
  <Provider store={store}>
    <ConnectedRouter history={history}>
      <Router />
    </ConnectedRouter>
  </Provider>,
  document.getElementById('root'),
)

registerServiceWorker()
