export const httpApi = Symbol('call ajax api')

/**
  * http请求redux中间件
  */
// eslint-disable-next-line no-unused-vars
const middleware = store => next => (action) => {
  const httpSymbol = action[httpApi]
  if (typeof httpSymbol === 'undefined') {
    return next(action)
  }
  const { options, types, url } = httpSymbol

  if (typeof url !== 'string') {
    throw new Error('Please specify a string URL.')
  }
  if (!Array.isArray(types) || !types.every(x => typeof x === 'string')) {
    throw new Error('Please specify a array types. Each of which has to be string.')
  }

  const actionWith = obj => ({
    ...action,
    [httpApi]: undefined,
    requestInformation: httpSymbol,
    ...obj,
  })
  // {string[]} http请求的四种状态处理类型, [请求成功，请求发起，请求失败，请求出错]
  const [
    successType,
    requestType = 'HTTP_REQUEST',
    failureType = 'HTTP_FAILURE',
    errorType = 'HTTP_ERROR',
  ] = types
  next(actionWith({ type: requestType }))
  // action.options fetch请求参数
  return fetch(url, options)
    .then((response) => {
      if (response.resultCode === '000000') {
        return next(actionWith({
          type: successType,
          response,
        }))
      }
      return next(actionWith({
        response,
        type: failureType,
      }))
    })
    .catch((error) => {
      next(actionWith({
        error,
        type: errorType,
      }))
      throw error
    })
}

export default middleware
