/**
 * webpack插件，需要与HtmlWebpackPlugin插件一同使用
 * https://github.com/ampedandwired/html-webpack-plugin#events
 * 用来往`index.html`插入变量，⚠️变量外部两边加百分号.
 * 示例: `new InterpolateHtmlPlugin({ 'MY_VARIABLE': 42 })`, 然后就可以在`index.html`使用%MY_VARIABLE%.
 */

'use strict' // eslint-disable-line

// 转义字符加反斜线，使得字符串可以作为RegExp参数
const escapeStringRegexp = require('escape-string-regexp')

class InterpolateHtmlPlugin {
  constructor(replacements) {
    this.replacements = replacements
  }

  apply(compiler) {
    compiler.plugin('compilation', (compilation) => {
      compilation.plugin(
        'html-webpack-plugin-before-html-processing',
        (data, callback) => {
          // Run HTML through a series of user-specified string replacements.
          Object.keys(this.replacements).forEach((key) => {
            const value = this.replacements[key]
            data.html = data.html.replace( // eslint-disable-line no-param-reassign
              new RegExp(`%${escapeStringRegexp(key)}%`, 'g'),
              value,
            )
          })
          callback(null, data)
        },
      )
    })
  }
}

module.exports = InterpolateHtmlPlugin
