/* eslint-disable import/no-unresolved */
'use strict' // eslint-disable-line

const ExtractTextPlugin = require('extract-text-webpack-plugin')

const notProduction = process.env.NODE_ENV !== 'production'

// 是否采用css-modules, 决定于环境变量CSS_MODULES
const shouldUseSourceMap = process.env.GENERATE_SOURCEMAP !== 'false' && notProduction

/**
 * get css webpack module rule
 * @param  {boolean} shouldUseCssModules      should use css modules
 * @param  {Object} extractTextPluginOptions
 * @return {Object}
 */
function getStyleRule(shouldUseCssModules, extractTextPluginOptions) {
  const loaders = [
    {
      // postcss不支持presets或者extends的扩展形式
      loader: require.resolve('postcss-loader'),
      options: {
        ident: 'postcss',
        config: {
          path: require.resolve('postcss-config-ivan'),
        },
        sourceMap: shouldUseSourceMap,
      },
    },
    {
      loader: require.resolve('sass-loader'),
      options: {
        sourceMap: shouldUseSourceMap,
      },
    },
    {
      loader: require.resolve('less-loader'),
      options: {
        sourceMap: shouldUseSourceMap,
      },
    },
  ]

  if (notProduction) {
    return [
      require.resolve('style-loader'),
      {
        loader: require.resolve('css-loader'),
        options: {
          sourceMap: shouldUseSourceMap,
        },
      },
    ].concat(loaders)
  }

  return ExtractTextPlugin.extract(Object.assign(
    {
      fallback: {
        loader: require.resolve('style-loader'),
        options: {
          hmr: false,
        },
      },
      use: [{
        loader: require.resolve('css-loader'),
        options: shouldUseCssModules ? {
          minimize: true,
          sourceMap: shouldUseSourceMap,
          modules: true,
          localIdentName: '[name]-[local]-[hash:base64:5]',
        } : {
          importLoaders: 3,
          minimize: true,
          sourceMap: shouldUseSourceMap,
        },
      }].concat(loaders),
    },
    extractTextPluginOptions,
  ))
}

module.exports = getStyleRule
