'use strict' // eslint-disable-line

const chalk = require('chalk')
const execSync = require('child_process').execSync // eslint-disable-line prefer-destructuring
const spawn = require('cross-spawn')
const opn = require('opn')

// 主要检测当前浏览器是否有已经存在的tab或者已经打开的对应窗口
// See https://github.com/sindresorhus/opn#app for documentation.
const OSX_CHROME = 'google chrome'

// 执行node脚本
function executeNodeScript(scriptPath, url) {
  const extraArgs = process.argv.slice(2)
  const child = spawn('node', [scriptPath, ...extraArgs, url], {
    stdio: 'inherit',
  })
  child.on('close', (code) => {
    if (code !== 0) {
      console.log(chalk.red('\nThe script specified as BROWSER environment variable failed.'))
      console.log(`${chalk.cyan(scriptPath)} exited with code ${code}.\n`)
    }
  })
  return true
}

// 在MacOS系统中浏览器chrome可以通过执行osascript脚本来进行
function startBrowserProcess(browser, url) {
  const shouldTryOpenChromeWithAppleScript =
    process.platform === 'darwin' &&
    (typeof browser !== 'string' || browser === OSX_CHROME)
  if (shouldTryOpenChromeWithAppleScript) {
    try {
      execSync('ps cax | grep "Google Chrome"')
      // 执行openChrome.applescript文件
      execSync(`osascript openChrome.applescript "${encodeURI(url)}"`, {
        cwd: __dirname, // 文件当前目录
        stdio: 'ignore',
      })
      return true
    } catch (err) {
      // Ignore errors.
    }
  }

  if (process.platform === 'darwin' && browser === 'open') {
    browser = undefined
  }

  try {
    const options = { app: browser }
    opn(url, options).catch(() => {}) // Prevent `unhandledRejection` error.
    return true
  } catch (err) {
    return false
  }
}

function openBrowser(url) {
  const value = process.env.BROWSER
  if (typeof value === 'string' && value.toLowerCase().endsWith('.js')) {
    return executeNodeScript(value, url)
  } else if (typeof value === 'string' && value.toLowerCase() === 'none') {
    // 环境变量BROWSER==="none"即不打开浏览器
    return false
  }
  return startBrowserProcess(value, url)
}

module.exports = openBrowser
