'use strict' // eslint-disable-line

const path = require('path')
const fs = require('fs')

// 项目文件夹
const appDirectory = fs.realpathSync(process.cwd())
const resolveApp = relativePath => path.resolve(appDirectory, relativePath)

// 按需求给path增加斜线
function ensureSlash(p, needsSlash) {
  const hasSlash = p.endsWith('/')
  if (hasSlash && !needsSlash) {
    return p.substr(p, p.length - 1)
  } else if (!hasSlash && needsSlash) {
    return `${p}/`
  }
  return p
}

// env.js预先执行时对环境变量PUBLIC_URL进行了设置
const envPublicUrl = process.env.PUBLIC_URL || '/'

const appHtml = resolveApp(process.env.APP_HTML || 'public/index.html')

const appIndexJs = resolveApp(process.env.APP_INDEXJS || 'src/index.js')

const checkRequiredFiles = require('./checkRequiredFiles')
// 检测入口文件是否存在
if (!checkRequiredFiles([appHtml, appIndexJs])) {
  process.exit(1)
}

module.exports = {
  appBuild: resolveApp(process.env.APP_BUILD || 'build'),
  appHtml: appHtml, // eslint-disable-line object-shorthand
  appIndexJs: appIndexJs, // eslint-disable-line object-shorthand
  appPackageJson: resolveApp('package.json'),
  appPublic: resolveApp(process.env.APP_PUBLIC || 'public'),
  appSrc: resolveApp(process.env.APP_SRC || 'src'),
  appNodeModules: resolveApp('node_modules'),
  servedPath: ensureSlash(envPublicUrl, true),
}
