/* eslint-disable import/no-unresolved */
'use strict' // eslint-disable-line

const path = require('path')

const webpack = require('webpack')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const CaseSensitivePathsPlugin = require('case-sensitive-paths-webpack-plugin')
const eslintFormatter = require('eslint-friendly-formatter')
const NpmInstallPlugin = require('npm-install-webpack-plugin')
const StyleLintPlugin = require('stylelint-webpack-plugin')

const InterpolateHtmlPlugin = require('./utils/InterpolateHtmlPlugin')
const ModuleScopePlugin = require('./utils/ModuleScopePlugin')
const getStyleRule = require('./utils/getStyleRule')

// 这个地方先引入env.js文件，在env.js文件中引入了paths.js，paths.js中各个变量是还未执行环境变量得到的
// 然后在env.js中delete掉require.cache上的paths.js缓存，
// 继而在下面第二行再度获取执行paths.js时，paths.js的代码是受到env环境变量的影响的
const getClientEnvironment = require('./utils/env')
const paths = require('./utils/paths')

// 是否采用css-modules, 决定于环境变量CSS_MODULES
const shouldUseCssModules = process.env.CSS_MODULES === 'true'

// webpack设置`publicPath参数`决定获取资源的公开URL。
// 开发环境中简化设置为根目录
const publicPath = '/'

// 对publicUrl值进行覆盖，开发环境中设为空字符串，用来在html模版代码中替换PUBLIC_URL
const publicUrl = ''
const env = getClientEnvironment(publicUrl)

module.exports = {
  devtool: 'cheap-module-source-map',
  entry: [
    require.resolve('./utils/polyfills'),
    require.resolve('webpack-dev-server/client') + '?/', // eslint-disable-line
    require.resolve('webpack/hot/dev-server'),
    paths.appIndexJs,
  ],
  output: {
    pathinfo: true,
    filename: 'static/js/bundle.js',
    chunkFilename: 'static/js/[name].chunk.js',
    publicPath,
    devtoolModuleFilenameTemplate: info => path.resolve(info.absoluteResourcePath).replace(/\\/g, '/'),
  },
  resolve: {
    modules: ['node_modules', paths.appNodeModules].concat(process.env.NODE_PATH.split(path.delimiter).filter(Boolean)),
    extensions: ['.web.js', '.mjs', '.js', '.json', '.web.jsx', '.jsx'],
    plugins: [
      // 限制从src或者node_modules目录之外引入文件
      new ModuleScopePlugin(paths.appSrc, [paths.appPackageJson]),
    ],
  },
  module: {
    strictExportPresence: true,
    rules: [
      {
        test: /\.(js|jsx|mjs)$/,
        enforce: 'pre',
        include: paths.appSrc,
        use: [{
          loader: require.resolve('eslint-loader'),
          options: {
            formatter: eslintFormatter,
            eslintPath: require.resolve('eslint'),
          },
        }],
      },
      {
        oneOf: [
          {
            test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
            loader: require.resolve('url-loader'),
            options: {
              limit: 10000,
              name: 'static/media/[name].[hash:8].[ext]',
            },
          },
          {
            test: /\.(js|jsx|mjs)$/,
            include: paths.appSrc, // 注意paths.appSrc外部的代码并未进行babel转换
            use: [{
              loader: require.resolve('babel-loader'),
              options: {
                cacheDirectory: true,
              },
            }],
          },
          {
            test: /\.(css|less|sass|scss)$/,
            exclude: [paths.appNodeModules, path.resolve(paths.appSrc, 'assets')],
            use: getStyleRule(shouldUseCssModules),
          },
          {
            test: /\.(css|less|sass|scss)$/,
            include: [paths.appNodeModules, path.resolve(paths.appSrc, 'assets')],
            use: getStyleRule(false),
          },
          {
            // Rule.oneOf无匹配时匹配最后一个loader，注意file-loader放最后
            exclude: [/\.js$/, /\.html$/, /\.json$/],
            loader: require.resolve('file-loader'),
            options: {
              name: 'static/media/[name].[hash:8].[ext]',
            },
          },
        ],
      },
    ],
  },
  plugins: [
    // html中插入模版代码
    new InterpolateHtmlPlugin(env.raw),
    // 用生成的script标签插入到index.html中
    new HtmlWebpackPlugin({
      inject: true,
      template: paths.appHtml,
    }),
    // 开启HMR时显示模块的相对路径
    new webpack.NamedModulesPlugin(),
    // 允许创建一个在**编译**时可以配置的全局常量。这可能会对开发模式和发布模式的构建允许不同的行为非常有用
    new webpack.DefinePlugin(env.stringified),
    // HMR
    new webpack.HotModuleReplacementPlugin(),
    // 文件大小写对应
    new CaseSensitivePathsPlugin(),
    // moment 2.18 会将所有本地化内容和核心功能一起打包
    new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
    // 自动安装并保存依赖包
    new NpmInstallPlugin(),
    // style lint 检查
    new StyleLintPlugin({
      files: [`${paths.appSrc}/**/*.css`],
      failOnError: false,
    }),
  ],
  node: {
    dgram: 'empty',
    fs: 'empty',
    net: 'empty',
    tls: 'empty',
    child_process: 'empty',
  },
  performance: {
    hints: false,
  },
}
